(function ($) {

Drupal.behaviors.map = {
  attach: function(context, settings) {

    var maps = settings.jsmaps;

    for(var map_i in maps){
    	var map = maps[map_i];
    	
    	//skip this map if the container div doesn't exist
    	if($('#' + map.jsmap_id,context).length == 0) continue;
    	
    	//create map canvas and regionset to contain everything that's drawn
    	var map_div = $('#' + map.jsmap_id,context);
    	var r = Raphael(map.jsmap_id, 1500, 1500);
    	map.regionset = r.set();//all regions
    	map.mapset = r.set();//everything drawn on the canvas
    	map.labelset = r.set();//just the label elements
    	map.lineset = r.set();//just the line elements
	    
	    //loop through all regions
	    for (var country in map.regions) {
	      var regiongroup = r.set();//all elements pertaining to this region (the region, text, and line)
	    	
	      //draw region and add it to the set
	      var obj = r.path(map.regions[country].path);
	      map.regionset.push(obj);
	      map.mapset.push(obj);
	      regiongroup.push(obj);
	
	      //set path attributes
	      obj.attr(map.defaults.pathDefaults);
	      if(map.defaults.pathDefaults['stroke-width'] == "0"){
	    	  obj.attr({'stroke-opacity': 0}); //must do this else chrome still draws the stroke https://github.com/DmitryBaranovskiy/raphael/issues/154
	      }
	      
	      //set the background of the region -- prefer the data-driven background, fall back to default
	      var regionBG = map.regions[country].fill || map.defaults.pathDefaults.fill;
	      obj.attr({'fill': regionBG});
	      
	      //decide if we should show labels on the map
	      var showNames = map.defaults.general.showNames;
	      if(showNames == 2){
	    	  showNames = map.regions[country].name_display;
	      }
	      showNames = (showNames == 1);
	      
	      //decide if the region should be interactive
	      var interactive = map.defaults.pathDefaults.interactive;
	      if(interactive == 2){
	    	  interactive = map.regions[country].interactive;
	      }
	      interactive = (interactive == 1);
	      
	      // draw name
	      if (showNames) {
	    	  //get the current region's bounding box
		      var bbox = obj.getBBox();
		      
	    	  var nameOffset = map.regions[country].name_offset ? map.regions[country].name_offset.split(' ') : [0,0];
			  var namePosX = bbox.x + bbox.width/2 + parseFloat(nameOffset[0]);
			  var namePosY = bbox.y + bbox.height/2 + parseFloat(nameOffset[1]);		
			  var text = r.text(namePosX, namePosY, map.regions[country].name);
			  text.attr({"font-size": map.defaults.general.fontSize});
			  map.labelset.push(text);
			  map.mapset.push(text);
			  regiongroup.push(text);
			    
			  // draw line to name, if appropriate
			  if (parseInt(map.regions[country].name_use_line)) {
				  var lineOffset = map.regions[country].name_line_offset ? map.regions[country].name_line_offset.split(' ') : [0,0];
			      var linePosX = bbox.x + bbox.width/2 + parseFloat(lineOffset[0]);
			      var linePosY = bbox.y + bbox.height/2 + parseFloat(lineOffset[1]);
			      var line = r.path('m '+ linePosX +' '+ linePosY +' l '+ (bbox.x - linePosX - 2) +' '+ ((bbox.y + bbox.height/2) - linePosY));
			      map.mapset.push(line);
			      map.lineset.push(line);
			      regiongroup.push(line);
			  }
			}
	      
	      if (interactive) {
	    	(function(){
	    		var c = country;
	    		var m = map;
	    		var m_d = map_div;
	    		var local_regionBG = regionBG;
	    		var region = obj;
	    		var t = text;
	    		var l = line;
	    		var data_position = map.defaults.general.data_position;
	    		
	    		var data_div = $('#' + m.regions[c].data_div_id,context).hide();//get the corresponding data div
	    		
	    		m_d.mouseleave(function(){
  	    		data_div.stop(true,true);
  	    		$('.map-starter-info').stop(true,true);
	    		  $('.jsmap-wrapper').css("cursor","auto");
	    			$('.mapDataBackground').stop(true,true).fadeOut(250,function(){
	    				$(this).remove();
	    				$('#views_slideshow_cycle_teaser_section_homepage_slideshow-block').cycle('resume');
	    			});
	    		});
	    		
	    		//make region go to link on click
	    		regiongroup.click(function() {
		          window.location = m.regions[c].url;
		      });
	    		
	    		//change color on hover and add popup to page
	    		regiongroup.hover(function(e){
	    			data_div.stop(true,true);
	    			region.stop(true,true);
	    			if(data_position == 'above'){
		    			var bbox = region.getBBox();
			    		data_div.css({position: 'absolute',left: (bbox.x + (bbox.width - data_div.outerWidth())/2), top: (bbox.y - data_div.outerHeight())});//position the data div centered above the shape
	    			}
	    			data_div.fadeIn(250);
	    			region.animate({
						fill: m.defaults.pathDefaults['fill-hover']
					}, 250);
								
				}, function(e){
					data_div.stop(true,true);
					region.stop(true,true);
					data_div.fadeOut(250);
					region.animate({
						fill: local_regionBG
					}, 250);
		        });
	    	})();
	        
	      }
	    }
	    
	    
	    //get the entire map's bounding box
	    var bbox = map.mapset.getBBox();
	    
	    var max_x = (bbox.x + bbox.width);
	    var max_y = (bbox.y + bbox.height);
	    var min_x = bbox.x;
	    var min_y = bbox.y;
	    var height = bbox.height;
	    var width = bbox.width;
	    var scale = map.defaults.general.scale;
	    var offset_x = 0;
	    var offset_y = 0;
	    
	    if(!(scale > 0)){
	    	var target_height = Number(map.defaults.general.max_height);
	    	var target_width = Number(map.defaults.general.max_width);
	    	var map_ratio = width / height;
	    	var target_ratio = target_width / target_height;
	    	
	    	//map is wider than target so base scale on width
	    	if(map_ratio > target_ratio){
	    		scale = target_width / width;
	    		offset_y = ((target_height / scale) - height) / 2;//center the map vertically
	    	}
	    	//map is taller than target so  base scale on height
	    	else{
	    		scale = target_height / height;
	    		offset_x = ((target_width / scale) - width) / 2;//center the map horizontally
	    	}
	    }
	    else{
	    	var target_height = height * scale;
	    	var target_width = width * scale;
	    }
	    
	    //scale everything
	    map.mapset.scale(scale, scale, 0, 0);
	    
	    //move the elements to the top left portion of the canvas so that all are visible
	    var dx = 0 - min_x + offset_x;
	    var dy = 0 - min_y + offset_y;
	    map.mapset.translate(dx, dy);
	    
	    //resize the canvas to just fit the contents
	    r.setSize(target_width + 2,target_height + 2);
	    	    
	    //move the regions to the back
	    map.regionset.toBack();
    }
  },

  detach: function(context, settings) {

  }
}

}(jQuery));
